"""Simple water flow example using ANUGA
Water flowing down a channel with a floodplain
"""

#------------------------------------------------------------------------------
# Import necessary modules
#------------------------------------------------------------------------------
# Import standard shallow water domain and standard boundaries.
import anuga
import numpy
from anuga.structures.inlet_operator import Inlet_operator

#from anuga import *
#from balanced_basic import *
from balanced_dev import *
#from balanced_basic.swb2_domain import *
#from balanced_basic.swb2_domain import Domain
#------------------------------------------------------------------------------
# Useful parameters for controlling this case
#------------------------------------------------------------------------------

floodplain_length = 1000.0 # Model domain length
floodplain_width = 14.0 # Model domain width
floodplain_slope = 1./300.
chan_initial_depth = 0.8 # Initial depth of water in the channel
chan_bankfull_depth = 1.0 # Bankfull depth of the channel
chan_width = 10.0 # Bankfull width of the channel
bankwidth = 2. # Width of the bank regions -- note that these protrude into the channel
man_n=0.03 # Manning's n
l0 = 2.009 # Length scale associated with triangle side length in channel (min_triangle area = 0.5*l0^2)

assert chan_width < floodplain_width, \
        ' ERROR: Channel width is greater than floodplain width'

assert bankwidth < chan_width/2., \
        'ERROR: The bank width must be less than half the channel width'

#------------------------------------------------------------------------------
# Setup computational domain
#------------------------------------------------------------------------------

# Define boundary polygon -- in this case clockwise around the proposed boundary
boundary_polygon = [ [0.,0.], 
                     [0., floodplain_length], 
                     [floodplain_width/2. - chan_width/2., floodplain_length], 
                     [floodplain_width/2. + chan_width/2., floodplain_length], 
                     [floodplain_width, floodplain_length], 
                     [floodplain_width, 0.], 
                     [floodplain_width/2. + chan_width/2., 0.], 
                     [floodplain_width/2. - chan_width/2., 0.] 
                     ]

# Define channel polygon, where we can optionally refine the resolution. 
# Note that we set this a distance l0 inside the boundary polygon, so the polygons
# do not overlap. 
channel_polygon = [ [floodplain_width/2. - chan_width/2., +l0],
                    [floodplain_width/2. - chan_width/2., floodplain_length-l0],
                    [floodplain_width/2. + chan_width/2., floodplain_length-l0],
                    [floodplain_width/2. + chan_width/2., +l0]
                    ]


# Define domain with appropriate boundary conditions
domain = create_domain_from_regions( boundary_polygon, 
                                   boundary_tags={'left': [0],
                                                  'top1': [1],
                                                  'chan_out': [2],
                                                  'top2': [3],
                                                  'right': [4],
                                                  'bottom1': [5],
                                                  'chan_in': [6],
                                                  'bottom2': [7] },
                                   maximum_triangle_area = 0.5*l0*l0,
                                   minimum_triangle_angle = 28.0,
                                   mesh_filename = 'channel_floodplain1.msh',
                                   interior_regions = [ ],
                                   #interior_regions = [\
                                   #    [channel_polygon, 0.5*l0*l0] ],
                                   use_cache=False,
                                   verbose=True)


domain.set_name('channel_floodplain1_test') # Output name
domain.extrapolate_velocity_second_order=True
#------------------------------------------------------------------------------
#
# Setup initial conditions
#
#------------------------------------------------------------------------------

# Function for topography
def topography(x, y):
    # Longitudinally sloping floodplain with channel in centre
	#return -y*floodplain_slope -chan_bankfull_depth*\
    #        (x>(floodplain_width/2. - chan_width/2.) )*\
    #        (x<(floodplain_width/2. + chan_width/2.) ) 

    elev1= -y*floodplain_slope - chan_bankfull_depth*\
            (x>(floodplain_width/2. - chan_width/2.))*\
            (x<(floodplain_width/2. + chan_width/2.)) 
    # Add banks
    if(bankwidth>0.0):
        leftbnk = floodplain_width/2. - chan_width/2.
        rightbnk = floodplain_width/2. + chan_width/2.
        # Left bank
        elev2 = elev1 + (chan_bankfull_depth \
                - chan_bankfull_depth/bankwidth*(x - leftbnk))*\
                (x>leftbnk)*(x < leftbnk + bankwidth)
        # Right bank
        elev2 = elev2 + (chan_bankfull_depth \
                + chan_bankfull_depth/bankwidth*(x - rightbnk))*\
                (x>rightbnk-bankwidth)*(x < rightbnk)
    
    if(bankwidth==0.0):
        elev2 = elev1
    #
    return elev2

#

#Function for stage
def stagetopo(x,y):
    return -y*floodplain_slope -chan_bankfull_depth + chan_initial_depth 

domain.set_quantity('elevation', topography) # Use function for elevation
domain.set_quantity('friction', man_n) # Constant friction
domain.set_quantity('stage', stagetopo) # Use function for stage
domain.set_minimum_allowed_height(0.01) # 
#def steady_uh_calc():
#    # Calculate a boundary condition potentially suitable for steady state flows.
#    uh = (chan_initial_depth**(4./3.)*man_n**(-2.)*floodplain_slope)**0.5\
#            *chan_initial_depth  # (u) * h
#    return uh
#
#momin = steady_uh_calc()
#velin = momin/chan_initial_depth
#
#print 'momentum inflow is = ' + str(momin)

# Define inlet operator 
if True:
    line1 = [ [floodplain_width/2. - chan_width/2., 0.],\
              [floodplain_width/2. + chan_width/2., 0.] \
              ]
    Qin = 0.5*(floodplain_slope*(chan_width*chan_initial_depth)**2.*man_n**(-2.)\
            *chan_initial_depth**(4./3.) )**0.5
    
    Inlet_operator(domain, line1, Qin)
    
    print 'Discharge in = ', Qin #,'Velocity at inlet should be ', Qin/(chan_width*chan_initial_depth), \
            #'for rectangular channels, and approximately that for trapezoidal ones'
#------------------------------------------------------------------------------
#
# Setup boundary conditions
#
#------------------------------------------------------------------------------

# We wish to define one inflow boundary at the upstream head of the channel,
# one at the outflow of the channel (e.g. transmissive)
# one at the floodplain edges, excluding the downstream floodplain edge
#(e.g. reflective)  and one at the downstream floodplain edge (e.g. transmissive)

#Bin_sub = anuga.shallow_water.boundaries.Dirichlet_discharge_boundary(domain,\
#    chan_initial_depth - chan_bankfull_depth , 0.) # An inflow for subcritical
#flow
#Bin_sup = anuga.shallow_water.boundaries.Dirichlet_discharge_boundary(domain, \
#        chan_initial_depth - chan_bankfull_depth , steady_uh_calc()) # An inflow  
#                                                                     # for 
#                                                                     #supercritical 
#                                                                     # flow
#Bin_sub = anuga.Dirichlet_boundary([chan_initial_depth - chan_bankfull_depth , \
#    0., 0.]) # An inflow for subcritical flow -- maybe should use transmissive
             # boundary instead
#Bin_sup = anuga.Dirichlet_boundary([chan_initial_depth - chan_bankfull_depth ,\
#        0., steady_uh_calc()]) # An inflow for supercritical flow

# Trial a novel boundary, which variously imposes 2 or 3 conserved variables
# depending on the local characteristic speed
#Bin_trial = anuga.shallow_water.boundaries.\
#        Dirichlet_inflow_boundary_sub_or_super_critical(domain, \
#                stage0=chan_initial_depth - chan_bankfull_depth, wh0 = momin,\
#                href = chan_initial_depth)

# Trial a set_stage_transmissive_momentum inflow boundary
#def inflow_stage_boundary(t):
#    return chan_initial_depth - chan_bankfull_depth

# transmissive_momentum_set_stage
#Bin_tmss = anuga.shallow_water.boundaries.Transmissive_momentum_set_stage_boundary(domain=domain, function = inflow_stage_boundary) 

Br = anuga.Reflective_boundary(domain) # Solid reflective wall
Bt = anuga.Transmissive_boundary(domain) # Transmissive boundary
#Bout_sub = anuga.Dirichlet_boundary( \
#        [-floodplain_length*floodplain_slope - chan_bankfull_depth + \
#        chan_initial_depth, 0., 0.]) #An outflow boundary for subcritical steady flow

def outflow_stage_boundary(t):
    return -floodplain_length*floodplain_slope \
            + chan_initial_depth - chan_bankfull_depth

# Note that the outflow boundary may be slightly incorrect for the trapezoidal channel case, 
# or incorrect more generally if there are numerical problems. But, in the central regions of
# the channel, this shouldn't really prevent us reaching steady, uniform flow.
Bout_tmss = anuga.shallow_water.boundaries.Transmissive_momentum_set_stage_boundary(domain, function = outflow_stage_boundary) 

domain.set_boundary({'left': Br, 
                     'right': Br, 
                     'top1': Bt, 
                     'top2': Bt, 
                     'bottom1': Br, 
                     'bottom2': Br, 
                     'chan_out': Bout_tmss, 
                     'chan_in': Br})


# To monitor the boundary conditions, let's find centroid points representing sub and super critical flows.
#inpt = numpy.argmin( ( abs(domain.centroid_coordinates[:,0] - floodplain_width/2.) + \
#        abs(domain.centroid_coordinates[:,1] - 0.0)  ) )
#outpt = numpy.argmin( ( abs(domain.centroid_coordinates[:, 0] - floodplain_width/2.) +\
#        abs(domain.centroid_coordinates[:, 1] - floodplain_length)  ) )

#------------------------------------------------------------------------------
#
# Evolve system through time
#
#------------------------------------------------------------------------------

for t in domain.evolve(yieldstep=1.0, finaltime=800.0):
    print domain.timestepping_statistics()
    #print domain.quantities['ymomentum'].get_integral()
    #print 'Qin = ', Qin
    #print domain.quantities['stage'].centroid_values[inpt]
    #print domain.quantities['xmomentum'].centroid_values[inpt]
    #print domain.quantities['ymomentum'].centroid_values[inpt]

    # Alter inflow boundary for sub and super critical flow
    # FIXME: Varying the inflow boundary conditions does not work well with the modified ANUGA 
    # (but works great with the standard ANUGA). Especially the supercritical inflow seems bad.
    # With the modified ANUGA, they oscillate (I wonder if the results are better with a finer grid?) 
    #h= domain.quantities['stage'].centroid_values[inpt] - domain.quantities['elevation'].centroid_values[inpt]
    #uh=domain.quantities['xmomentum'].centroid_values[inpt]  
    #if( domain.g*h < (uh/h)**2):
    #    domain.set_boundary({'chan_in': Bin_sup})
    #else:
    #    domain.set_boundary({'chan_in': Bin_sub})

    # Alter outflow boundary for sub and super critical flow
    #h=domain.quantities['stage'].centroid_values[outpt] - domain.quantities['elevation'].centroid_values[outpt]
    #uh=domain.quantities['xmomentum'].centroid_values[outpt]  
    #if( domain.g*h < (uh/h)**2):
    #    domain.set_boundary({'chan_out': Bt})
    #else:
    #    domain.set_boundary({'chan_out': Bout_sub})

